@@ -29,8 +29,9 @@
 #include <KLocale>
 #include <QDir>
 #include <QTimer>
+#include <QMutexLocker>
 #include <language/duchain/topducontext.h>
-#include <language/interfaces/iproblem.h>
+#include <language/duchain/problem.h>
 #include <language/duchain/duchain.h>
 
 #include "python_header.h"
@@ -135,13 +136,30 @@ QPair<QString, int> fileHeaderHack(QString& contents, const KUrl& filename)
     }
 }
 
-AstBuilder::AstBuilder(KDevPG::MemoryPool* pool)
-    : m_pool(pool)
-{
+namespace {
+struct PythonInitializer : private QMutexLocker {
+    PythonInitializer(QMutex& pyInitLock, const QString& pyHomeDir):
+        QMutexLocker(&pyInitLock), arena(0)
+    {
+            Py_SetPythonHome(pyHomeDir.toUtf8().data());
+            Py_InitializeEx(0);
+            Q_ASSERT(Py_IsInitialized());
 
+            arena = PyArena_New();
+            Q_ASSERT(arena); // out of memory
+    }
+    ~PythonInitializer()
+    {
+        if (arena)
+            PyArena_Free(arena);
+        if (Py_IsInitialized())
+            Py_Finalize();
+    }
+    PyArena* arena;
+};
 }
 
-CodeAst* AstBuilder::parse(KUrl filename, QString& contents)
+CodeAst::Ptr AstBuilder::parse(KUrl filename, QString &contents)
 {
     qDebug() << " ====> AST     ====>     building abstract syntax tree for " << filename.path();
     
@@ -152,22 +170,16 @@ CodeAst* AstBuilder::parse(KUrl filename, QString& contents)
     QPair<QString, int> hacked = fileHeaderHack(contents, filename);
     contents = hacked.first;
     int lineOffset = hacked.second;
-    
-    AstBuilder::pyInitLock.lock();
-    Py_SetPythonHome(AstBuilder::pyHomeDir.toUtf8().data());
-    kDebug() << "Not initialized, calling init func.";
-    Py_Initialize();
-    Q_ASSERT(Py_IsInitialized());
-    
-    PyArena* arena = PyArena_New();
-    Q_ASSERT(arena); // out of memory
-    PyCompilerFlags* flags = new PyCompilerFlags();
-    flags->cf_flags = PyCF_SOURCE_IS_UTF8;
-    
+
+    PythonInitializer pyIniter(pyInitLock, pyHomeDir);
+    PyArena* arena = pyIniter.arena;
+
+    PyCompilerFlags flags = {PyCF_SOURCE_IS_UTF8};
+
     PyObject *exception, *value, *backtrace;
     PyErr_Fetch(&exception, &value, &backtrace);
 
-    mod_ty syntaxtree = PyParser_ASTFromString(contents.toUtf8().data(), "<kdev-editor-contents>", file_input, flags, arena);
+    mod_ty syntaxtree = PyParser_ASTFromString(contents.toUtf8().data(), "<kdev-editor-contents>", file_input, &flags, arena);
 
     if ( ! syntaxtree ) {
         kWarning() << "DID NOT RECEIVE A SYNTAX TREE -- probably parse error.";
@@ -183,8 +195,7 @@ CodeAst* AstBuilder::parse(KUrl filename, QString& contents)
        
         if ( ! errorDetails_tuple ) {
             kWarning() << "Error retrieving error message, not displaying, and not doing anything";
-            pyInitLock.unlock();
-            return 0;
+            return CodeAst::Ptr();
         }
         PyObject* linenoobj = PyTuple_GetItem(errorDetails_tuple, 1);
         errorMessage_str = PyTuple_GetItem(value, 0);
@@ -275,8 +286,8 @@ CodeAst* AstBuilder::parse(KUrl filename, QString& contents)
                 break;
             }
         }
-        
-        syntaxtree = PyParser_ASTFromString(contents.toUtf8(), "<kdev-editor-contents>", file_input, flags, arena);
+
+        syntaxtree = PyParser_ASTFromString(contents.toUtf8(), "<kdev-editor-contents>", file_input, &flags, arena);
         // 3rd try: discard everything after the last non-empty line, but only until the next block start
         currentLineBeginning = qMin(contents.length() - 1, currentLineBeginning);
         errline = qMax(0, qMin(indents.length()-1, errline));
@@ -318,29 +329,21 @@ CodeAst* AstBuilder::parse(KUrl filename, QString& contents)
                 if ( c.isSpace() && atLineBeginning ) currentIndent += 1;
             }
             kDebug() << "This is what is left: " << contents;
-            syntaxtree = PyParser_ASTFromString(contents.toUtf8(), "<kdev-editor-contents>", file_input, flags, arena);
+            syntaxtree = PyParser_ASTFromString(contents.toUtf8(), "<kdev-editor-contents>", file_input, &flags, arena);
         }
         if ( ! syntaxtree ) {
-            PyArena_Free(arena);
-            Py_Finalize();
-            pyInitLock.unlock();
-            return 0; // everything fails, so we abort.
+            return CodeAst::Ptr(); // everything fails, so we abort.
         }
     }
     kDebug() << "Got syntax tree from python parser:" << syntaxtree->kind << Module_kind;
 
-    PythonAstTransformer t(lineOffset, m_pool);
+    PythonAstTransformer t(lineOffset);
     t.run(syntaxtree, filename.fileName().replace(".py", ""));
     
-    PyArena_Free(arena);
-    Py_Finalize();
-    
-    AstBuilder::pyInitLock.unlock();
-    
     RangeUpdateVisitor v;
     v.visitNode(t.ast);
 
-    return t.ast;
+    return CodeAst::Ptr(t.ast);
 }
 
 }
